@@ -265,8 +265,6 @@ static int _thin_pool_add_target_line(struct dev_manager *dm,
 	char *metadata_dlid, *pool_dlid;
 	const struct lv_thin_message *lmsg;
 	const struct logical_volume *origin;
-	struct lvinfo info;
-	uint64_t transaction_id = 0;
 	unsigned attr;
 	uint64_t low_water_mark;
 	int threshold;
@@ -351,21 +349,6 @@ static int _thin_pool_add_target_line(struct dev_manager *dm,
 		case DM_THIN_MESSAGE_CREATE_THIN:
 			origin = first_seg(lmsg->u.lv)->origin;
 			/* Check if the origin is suspended */
-			if (origin && lv_info(cmd, origin, 1, &info, 0, 0) &&
-			    info.exists && !info.suspended) {
-				/* Origin is not suspended, but the transaction may have been
-				 * already transfered, so test for transaction_id and
-				 * allow to pass in the message for dmtree processing
-				 * so it will skip all messages later.
-				 */
-				if (!lv_thin_pool_transaction_id(seg->lv, &transaction_id))
-					return_0; /* Thin pool should exist and work */
-				if ((transaction_id + 1) != seg->transaction_id) {
-					log_error("Can't create snapshot %s as origin %s is not suspended.",
-						  lmsg->u.lv->name, origin->name);
-					return 0;
-				}
-			}
 			log_debug_activation("Thin pool create_%s %s.", (!origin) ? "thin" : "snap", lmsg->u.lv->name);
 			if (!dm_tree_node_add_thin_pool_message(node,
 								(!origin) ? lmsg->type : DM_THIN_MESSAGE_CREATE_SNAP,
