@@ -193,11 +193,14 @@ def call_lvm(command):
 
 def supports_vdo():
 	cmd = ['segtypes']
+	modprobe = Popen(["modprobe", "kvdo"], stdout=PIPE, stderr=PIPE, close_fds=True, env=os.environ)
+	modprobe.communicate()
+	if modprobe.returncode != 0:
+		return False
 	rc, out, err = call_lvm(cmd)
-	if rc == 0:
-		if "vdo" in out:
-			return True
-	return False
+	if rc != 0 or "vdo" not in out:
+		return False
+	return True
 
 
 # noinspection PyUnresolvedReferences
@@ -1555,6 +1558,36 @@ class TestDbusService(unittest.TestCase):
 			cached_lv.Lv.Rename(dbus.String(new_name), dbus.Int32(g_tmo), EOD))
 		verify_cache_lv_count()
 
+	def test_writecache_lv(self):
+		vg = self._vg_create().Vg
+		data_lv = self._create_lv(size=mib(16), vg=vg)
+		cache_lv = self._create_lv(size=mib(16), vg=vg)
+
+		# both LVs need to be inactive
+		self.handle_return(data_lv.Lv.Deactivate(
+			dbus.UInt64(0), dbus.Int32(g_tmo), EOD))
+		data_lv.update()
+		self.handle_return(cache_lv.Lv.Deactivate(
+			dbus.UInt64(0), dbus.Int32(g_tmo), EOD))
+		cache_lv.update()
+
+		cached_lv_path = self.handle_return(
+			cache_lv.Lv.WriteCacheLv(
+				dbus.ObjectPath(data_lv.object_path),
+				dbus.Int32(g_tmo),
+				EOD))
+
+		intf = (LV_COMMON_INT, LV_INT, CACHE_LV_INT)
+		cached_lv = ClientProxy(self.bus, cached_lv_path, interfaces=intf)
+		self.assertEqual(cached_lv.LvCommon.SegType, ["writecache"])
+
+		uncached_lv_path = self.handle_return(
+				cached_lv.CachedLv.DetachCachePool(
+					dbus.Boolean(True),
+					dbus.Int32(g_tmo),
+					EOD))
+		self.assertTrue('/com/redhat/lvmdbus1/Lv' in uncached_lv_path)
+
 	def test_vg_change(self):
 		vg_proxy = self._vg_create()
 
@@ -1855,11 +1888,11 @@ class TestDbusService(unittest.TestCase):
 		self.assertEqual(pv_object_path, self._lookup(symlink))
 		self.assertEqual(pv_object_path, self._lookup(pv_device_path))
 
-	def _create_vdo_pool_and_lv(self):
+	def _create_vdo_pool_and_lv(self, vg_prefix="vdo_"):
 		pool_name = lv_n("_vdo_pool")
 		lv_name = lv_n()
 
-		vg_proxy = self._vg_create(vg_prefix="vdo_")
+		vg_proxy = self._vg_create(vg_prefix=vg_prefix)
 		vdo_pool_object_path = self.handle_return(
 			vg_proxy.VgVdo.CreateVdoPoolandLv(
 				pool_name, lv_name,
@@ -1900,6 +1933,72 @@ class TestDbusService(unittest.TestCase):
 			vg, _, _ = self._create_vdo_pool_and_lv()
 			self.handle_return(vg.Vg.Remove(dbus.Int32(g_tmo), EOD))
 
+	def _create_vdo_pool(self):
+		pool_name = lv_n('_vdo_pool')
+		lv_name = lv_n('_vdo_data')
+		vg_proxy = self._vg_create(vg_prefix="vdo_conv_")
+		lv = self._test_lv_create(
+			vg_proxy.Vg.LvCreate,
+			(dbus.String(pool_name), dbus.UInt64(mib(4096)),
+				dbus.Array([], signature='(ott)'), dbus.Int32(g_tmo),
+				EOD), vg_proxy.Vg, LV_BASE_INT)
+		lv_obj_path = self._lookup("%s/%s" % (vg_proxy.Vg.Name, pool_name))
+		self.assertNotEqual(lv_obj_path, "/")
+
+		vdo_pool_path = self.handle_return(
+			vg_proxy.VgVdo.CreateVdoPool(
+				dbus.ObjectPath(lv.object_path), lv_name,
+				dbus.UInt64(mib(8192)),
+				dbus.Int32(g_tmo),
+				EOD))
+
+		self.assertNotEqual(vdo_pool_path, "/")
+		self.assertEqual(
+			vdo_pool_path,
+			self._lookup("%s/%s" % (vg_proxy.Vg.Name, pool_name)))
+		intf = [LV_COMMON_INT, LV_INT]
+		vdo_lv_obj_path = self._lookup("%s/%s" % (vg_proxy.Vg.Name, lv_name))
+		vdo_lv = ClientProxy(self.bus, vdo_lv_obj_path, interfaces=intf)
+		intf.append(VDOPOOL_INT)
+		vdo_pool_lv = ClientProxy(self.bus, vdo_pool_path, interfaces=intf)
+		return vg_proxy, vdo_pool_lv, vdo_lv
+
+	def test_vdo_pool_convert(self):
+		# Basic vdo sanity testing
+		if not self.vdo:
+			raise unittest.SkipTest('vdo not supported')
+
+		vg, _pool, _lv = self._create_vdo_pool()
+		self.handle_return(vg.Vg.Remove(dbus.Int32(g_tmo), EOD))
+
+	def test_vdo_pool_compression_deduplication(self):
+		if not self.vdo:
+			raise unittest.SkipTest('vdo not supported')
+
+		vg, pool, _lv = self._create_vdo_pool_and_lv(vg_prefix="vdo2_")
+
+		# compression and deduplication should be enabled by default
+		self.assertEqual(pool.VdoPool.Compression, "enabled")
+		self.assertEqual(pool.VdoPool.Deduplication, "enabled")
+
+		self.handle_return(
+			pool.VdoPool.DisableCompression(dbus.Int32(g_tmo), EOD))
+		self.handle_return(
+			pool.VdoPool.DisableDeduplication(dbus.Int32(g_tmo), EOD))
+		pool.update()
+		self.assertEqual(pool.VdoPool.Compression, "")
+		self.assertEqual(pool.VdoPool.Deduplication, "")
+
+		self.handle_return(
+			pool.VdoPool.EnableCompression(dbus.Int32(g_tmo), EOD))
+		self.handle_return(
+			pool.VdoPool.EnableDeduplication(dbus.Int32(g_tmo), EOD))
+		pool.update()
+		self.assertEqual(pool.VdoPool.Compression, "enabled")
+		self.assertEqual(pool.VdoPool.Deduplication, "enabled")
+
+		self.handle_return(vg.Vg.Remove(dbus.Int32(g_tmo), EOD))
+
 	def _test_lv_method_interface(self, lv):
 		self._rename_lv_test(lv)
 		self._test_activate_deactivate(lv)
