@@ -123,6 +123,7 @@ static inline int reportformat_arg(struct cmd_context *cmd __attribute__((unused
 static inline int configreport_arg(struct cmd_context *cmd __attribute__((unused)), struct arg_values *av) { return 0; }
 static inline int configtype_arg(struct cmd_context *cmd __attribute__((unused)), struct arg_values *av) { return 0; }
 static inline int repairtype_arg(struct cmd_context *cmd __attribute__((unused)), struct arg_values *av) { return 0; }
+static inline int dumptype_arg(struct cmd_context *cmd __attribute__((unused)), struct arg_values *av) { return 0; }
 
 /* needed to include commands.h when building man page generator */
 #define CACHE_VGMETADATA        0x00000001
@@ -366,7 +367,7 @@ static int _opt_str_to_num(struct command *cmd, char *str)
 	int i;
 	int first = 0, last = ARG_COUNT - 1, middle;
 
-	dm_strncpy(long_name, str, sizeof(long_name));
+	(void) dm_strncpy(long_name, str, sizeof(long_name));
 
 	if ((p = strstr(long_name, "_long")))
 		/*
@@ -801,7 +802,7 @@ static char *_get_oo_line(const char *str)
 	char str2[OO_NAME_LEN];
 	int i;
 
-	dm_strncpy(str2, str, sizeof(str2));
+	(void) dm_strncpy(str2, str, sizeof(str2));
 	if ((end = strchr(str2, ':')))
 		*end = '\0';
 	if ((end = strchr(str2, ',')))
@@ -998,8 +999,8 @@ static void __add_optional_opt_line(struct cmd_context *cmdtool, struct command
 		else if (takes_arg)
 			_update_prev_opt_arg(cmdtool, cmd, argv[i], OPTIONAL);
 		else {
-			log_error("Parsing command defs: can't parse argc %d argv %s prev %s.",
-				i, argv[i], argv[i-1]);
+			log_error("Parsing command defs: can't parse argc %d argv %s%s%s.",
+				i, argv[i], (i > 0) ? " prev " : "", (i > 0) ? argv[i - 1] : "");
 			cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 			return;
 		}
@@ -1024,8 +1025,8 @@ static void _add_ignore_opt_line(struct cmd_context *cmdtool, struct command *cm
 		else if (takes_arg)
 			_update_prev_opt_arg(cmdtool, cmd, argv[i], IGNORE);
 		else {
-			log_error("Parsing command defs: can't parse argc %d argv %s prev %s.",
-				i, argv[i], argv[i-1]);
+			log_error("Parsing command defs: can't parse argc %d argv %s%s%s.",
+				  i, argv[i], (i > 0) ? " prev " : "", (i > 0) ? argv[i - 1] : "");
 			cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 			return;
 		}
@@ -1059,8 +1060,8 @@ static void _add_required_opt_line(struct cmd_context *cmdtool, struct command *
 		else if (takes_arg)
 			_update_prev_opt_arg(cmdtool, cmd, argv[i], REQUIRED);
 		else {
-			log_error("Parsing command defs: can't parse argc %d argv %s prev %s.",
-				  i, argv[i], argv[i-1]);
+			log_error("Parsing command defs: can't parse argc %d argv %s%s%s.",
+				  i, argv[i], (i > 0) ? " prev " : "", (i > 0) ? argv[i - 1] : "");
 			cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 			return;
 		}
@@ -1149,8 +1150,8 @@ static void _add_required_line(struct cmd_context *cmdtool, struct command *cmd,
 			/* set property for previous required_pos_arg */
 			_update_prev_pos_arg(cmd, argv[i], REQUIRED);
 		} else {
-			log_error("Parsing command defs: can't parse argc %d argv %s prev %s.",
-				  i, argv[i], argv[i-1]);
+			log_error("Parsing command defs: can't parse argc %d argv %s%s%s.",
+				  i, argv[i], (i > 0) ? " prev " : "", (i > 0) ? argv[i - 1] : "");
 			cmd->cmd_flags |= CMD_FLAG_PARSE_ERROR;
 			return;
 		}
@@ -1419,6 +1420,9 @@ int define_commands(struct cmd_context *cmdtool, const char *run_name)
 		if (line[0] == '\n')
 			break;
 
+		if (!strcmp(line, "---") || !strcmp(line, "--"))
+			continue;
+
 		if ((n = strchr(line, '\n')))
 			*n = '\0';
 
@@ -2318,7 +2322,8 @@ static void _print_val_man(struct command_name *cname, int opt_enum, int val_enu
 	}
 
 	if (strchr(str, '|')) {
-		line = strdup(str);
+		if (!(line = strdup(str)))
+			return;
 		_split_line(line, &line_argc, line_argv, '|');
 		for (i = 0; i < line_argc; i++) {
 			if (i)
@@ -3287,9 +3292,8 @@ static void _print_man_all_positions_desc(struct command_name *cname)
 	       "capitalization, e.g. 'k' and 'K' both refer to 1024.\n"
 	       "The default input unit is specified by letter, followed by |UNIT.\n"
 	       "UNIT represents other possible input units: \\fBbBsSkKmMgGtTpPeE\\fP.\n"
-	       "b|B is bytes, s|S is sectors of 512 bytes, k|K is kilobytes,\n"
-	       "m|M is megabytes, g|G is gigabytes, t|T is terabytes,\n"
-	       "p|P is petabytes, e|E is exabytes.\n"
+	       "b|B is bytes, s|S is sectors of 512 bytes, k|K is KiB,\n"
+	       "m|M is MiB, g|G is GiB, t|T is TiB, p|P is PiB, e|E is EiB.\n"
 	       "(This should not be confused with the output control --units, where\n"
 	       "capital letters mean multiple of 1000.)\n");
 
@@ -3605,9 +3609,12 @@ int main(int argc, char *argv[])
 		goto out_free;
 	}
 
-	if (optind < argc)
-		cmdname = strdup(argv[optind++]);
-	else {
+	if (optind < argc) {
+		if (!(cmdname = strdup(argv[optind++]))) {
+			log_error("Out of memory.");
+			goto out_free;
+		}
+	} else {
 		log_error("Missing command name.");
 		goto out_free;
 	}
